@@ -5,11 +5,11 @@ use warnings;
 package Sub::HandlesVia::Toolkit;
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.022';
+our $VERSION   = '0.023';
 
 use Type::Params qw(compile_named_oo);
-use Types::Standard qw( ArrayRef HashRef Str Num Int CodeRef Bool );
-use Types::Standard qw( assert_HashRef is_ArrayRef );
+use Types::Standard qw( ArrayRef HashRef Str Num Int CodeRef Bool Item );
+use Types::Standard qw( assert_HashRef is_ArrayRef is_CodeRef is_Str );
 
 my $sig;
 sub install_delegations {
@@ -37,11 +37,6 @@ sub install_delegations {
 			$arg->handles_via,
 		);
 		
-#		warn $handler->code_as_string(
-#			method_name => $h,
-#			code_generator => $gen,
-#		);
-
 		$handler->install_method(
 			method_name    => $h,
 			code_generator => $gen,
@@ -72,6 +67,7 @@ my %default_type = (
 	Counter   => Int,
 	Code      => CodeRef,
 	Bool      => Bool,
+	Scalar    => Item,
 );
 
 sub clean_spec {
@@ -116,12 +112,93 @@ sub clean_spec {
 
 sub code_generator_for_attribute {
 	my ($me, $target, $attr) = (shift, @_);
-	die "must be implemented by child classes";
+	
+	my ($get_slot, $set_slot, $default) = @$attr;
+	$set_slot = $get_slot if @$attr < 2;
+	
+	my $captures = {};
+	my ($get, $set, $get_is_lvalue) = (undef, undef, 0);
+	
+	require B;
+	
+	if (ref $get_slot) {
+		$get = sub { '$_[0]->$shv_reader' };
+		$captures->{'$shv_reader'} = \$get_slot;
+	}
+	elsif ($get_slot =~ /\A \[ ([0-9]+) \] \z/sx) {
+		my $index = $1;
+		$get = sub { "\$_[0][$index]" };
+		++$get_is_lvalue;
+	}
+	elsif ($get_slot =~ /\A \{ (.+) \} \z/sx) {
+		my $key = B::perlstring($1);
+		$get = sub { "\$_[0]{$key}" };
+		++$get_is_lvalue;
+	}
+	else {
+		my $method = B::perlstring($get_slot);
+		$get = sub { "\$_[0]->\${\\ $method}" };
+	}
+	
+	if (ref $set_slot) {
+		$set = sub { my $val = shift or die; "\$_[0]->\$shv_writer($val)" };
+		$captures->{'$shv_writer'} = \$set_slot;
+	}
+	elsif ($set_slot =~ /\A \[ ([0-9]+) \] \z/sx) {
+		my $index = $1;
+		$set = sub { my $val = shift or die; "(\$_[0][$index] = $val)" };
+	}
+	elsif ($set_slot =~ /\A \{ (.+) \} \z/sx) {
+		my $key = B::perlstring($1);
+		$set = sub { my $val = shift or die; "(\$_[0]{$key} = $val)" };
+	}
+	else {
+		my $method = B::perlstring($set_slot);
+		$set = sub { my $val = shift or die; "\$_[0]->\${\\ $method}($val)" };
+	}
+	
+	if (is_CodeRef $default) {
+		$captures->{'$shv_default_for_reset'} = \$default;
+	}
+
+	require Sub::HandlesVia::CodeGenerator;
+	return 'Sub::HandlesVia::CodeGenerator'->new(
+		toolkit               => $me,
+		target                => $target,
+		attribute             => $attr,
+		env                   => $captures,
+		coerce                => !!0,
+		generator_for_get     => $get,
+		generator_for_set     => $set,
+		get_is_lvalue         => $get_is_lvalue,
+		set_checks_isa        => !!1,
+		set_strictly          => !!1,
+		generator_for_default => sub {
+			my ( $gen, $handler ) = @_ or die;
+			if ( !$default and $handler ) {
+				return $handler->default_for_reset->();
+			}
+			elsif ( is_CodeRef $default ) {
+				return sprintf(
+					'(%s)->$shv_default_for_reset',
+					$gen->generator_for_self->(),
+				);
+			}
+			elsif ( is_Str $default ) {
+				require B;
+				return sprintf(
+					'(%s)->${\ %s }',
+					$gen->generator_for_self->(),
+					B::perlstring( $default ),
+				);
+			}
+			return;
+		},
+	);
 }
 
 1;
 
-
 __END__
 
 =pod
@@ -134,6 +211,9 @@ Sub::HandlesVia::Toolkit - integration with OO frameworks for Sub::HandlesVia
 
 =head1 DESCRIPTION
 
+B<< This module is part of Sub::HandlesVia's internal API. >>
+It is mostly of interest to people extending Sub::HandlesVia.
+
 Detect what subclass of Sub::HandlesVia::Toolkit is suitable for a class:
 
   my $toolkit = Sub::HandlesVia->detect_toolkit($class);
